#ifndef MSP_VR_STEREOVIEW_H_
#define MSP_VR_STEREOVIEW_H_

#include <msp/geometry/angle.h>
#include <msp/gl/camera.h>
#include <msp/gl/framebuffer.h>
#include <msp/gl/renderable.h>
#include <msp/gl/renderbuffer.h>
#include <msp/gl/texture2d.h>

namespace Msp {
namespace VR {

class HeadTrackingCamera;
class MotionController;
class StereoCombiner;

class StereoView
{
private:
	struct RenderTarget
	{
		GL::Framebuffer fbo;
		GL::Texture2D color;
		GL::Renderbuffer depth;

		RenderTarget(unsigned, unsigned);
	};

	struct EyeParams
	{
		Geometry::Angle<float> fov;
		float aspect;
		float near_clip;
		float far_clip;
	};

	struct Eye
	{
		mutable GL::Camera camera;
		GL::Matrix offset_matrix;
		RenderTarget *target;

		Eye();
		~Eye();

		void create_target(unsigned, unsigned);
		void setup_frame(const GL::Camera &, float, const EyeParams &) const;
		void render(const GL::Renderable &) const;
	};

	const StereoCombiner &combiner;
	const GL::Camera &base_camera;
	HeadTrackingCamera *head_camera;
	const GL::Renderable *content;
	Eye left;
	Eye right;
	Geometry::Angle<float> strabismus;
	std::vector<MotionController *> controllers;

public:
	StereoView(const StereoCombiner &, const GL::Camera &);
	StereoView(const StereoCombiner &, HeadTrackingCamera &);
private:
	void init();

public:
	const GL::Camera &get_base_camera() const { return base_camera; }
	const HeadTrackingCamera *get_head_camera() const { return head_camera; }
	void set_content(const GL::Renderable *);
	void set_eye_spacing(float);
	void set_eye_matrices(const GL::Matrix &, const GL::Matrix &);
	void set_strabismus(const Geometry::Angle<float> &);

	void add_controller(MotionController &);
	void remove_controller(MotionController &);

private:
	void setup_frame() const;
public:
	void render() const;
};

} // namespace VR
} // namespace Msp

#endif
